local addonName, addon = ...

local config = addon.config
local panel = addon.panel
local scrollFrame = addon.scrollFrame
local L = addon.language
--设置面板初始化
panel:SetSize(500, 1000)
scrollFrame.ScrollBar:ClearAllPoints()
scrollFrame.ScrollBar:SetPoint("TOPLEFT", scrollFrame, "TOPRIGHT", -20, -20)
scrollFrame.ScrollBar:SetPoint("BOTTOMLEFT", scrollFrame, "BOTTOMRIGHT", -20, 20)
scrollFrame:SetScrollChild(panel)
scrollFrame.name = addonName
InterfaceOptions_AddCategory(scrollFrame)
--标题
local title = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLargeLeft')
title:SetPoint('TOPLEFT', 16, -16)
title:SetText(addonName.." v"..config.version)
--所有组件表
panel.controls = {}
--所有音效列表
local soundList = {
	'', '关',
	'Sound\\Doodad\\BellTollAlliance.ogg', 'Bell 1',
	'Sound\\Doodad\\BellTollHorde.ogg', 'Bell 2',
	'Sound\\Doodad\\FX_Ice_Shard_Impact_03.OGG', 'Ice Shard',
	'Sound\\Doodad\\FX_MK_IMPACT_MED_03.OGG', 'Impact',
	'Sound\\Doodad\\BE_ScryingOrb_Explode.ogg', 'Orb Explode',
	'Sound\\SPELLS\\PetCall.ogg', 'Pet Call',
	'Sound\\character\\PlayerRoars\\CharacterRoarsDwarfMale.ogg', 'Roar Dwarf',
	'Sound\\character\\PlayerRoars\\CharacterRoarsOrcMale.ogg', 'Roar Orc',
	'Sound\\Doodad\\FX_GEN_KUNGFU_02.OGG', 'Swoosh',
	'Sound\\character\\EmoteCatCallWhistle05.ogg', 'Whistle',
	-- Custom sound files user can add to addon folder
	'Interface\\AddOns\\' .. addonName .. '\\sound1.mp3', 'Custom sound1.mp3',
	'Interface\\AddOns\\' .. addonName .. '\\sound2.mp3', 'Custom sound2.mp3',
	'Interface\\AddOns\\' .. addonName .. '\\sound1.ogg', 'Custom sound1.ogg',
	'Interface\\AddOns\\' .. addonName .. '\\sound2.ogg', 'Custom sound2.ogg',
}
--创建唯一命名函数
local UniqueName
do
	local controlID = 1

	function UniqueName(name)
		controlID = controlID + 1
		return string.format('%s_%s_%02d', addonName, name, controlID)
	end
end
--设置面板确定函数
function scrollFrame:ConfigOkay()
	for _, control in pairs(panel.controls) do
		control.SaveValue(control.currentValue)
	end
end
--设置面板回到默认设置函数
function scrollFrame:ConfigDefault()
	for _, control in pairs(panel.controls) do
		control.currentValue = control.defaultValue
		control.SaveValue(control.currentValue)
	end
end
--设置面板刷新函数
function scrollFrame:ConfigRefresh()
	for _, control in pairs(panel.controls) do
		control.currentValue = control.LoadValue()
		control:UpdateValue()
	end
end
--创建标题函数
function panel:CreateHeading(text)
	local title = self:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLeft')
	title:SetText(text)

	return title
end
--创建文本函数
function panel:CreateText(text)
	local blob = self:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmallLeft')
	blob:SetText(text)
	return blob
end
--创建选择框函数
function panel:CreateCheckBox(text, LoadValue, SaveValue, defaultValue)
	local checkBox = CreateFrame('CheckButton', UniqueName('CheckButton'), self, 'InterfaceOptionsCheckButtonTemplate')

	checkBox.LoadValue = LoadValue
	checkBox.SaveValue = SaveValue
	checkBox.defaultValue = defaultValue
	checkBox.UpdateValue = function(self) self:SetChecked(self.currentValue) end
	getglobal(checkBox:GetName() .. 'Text'):SetText(text)
	checkBox:SetScript('OnClick', function(self) self.currentValue = self:GetChecked() end)

	self.controls[checkBox:GetName()] = checkBox
	return checkBox
end
--下拉菜单点击函数
local function DropDownOnClick(_, dropDown, selectedValue)
	dropDown.currentValue = selectedValue
	UIDropDownMenu_SetText(dropDown, dropDown.valueTexts[selectedValue])
end
--下拉菜单初始化函数
local function DropDownInitialize(frame)
	local info = UIDropDownMenu_CreateInfo()

	for i=1,#frame.valueList,2 do
		local k, v = frame.valueList[i], frame.valueList[i + 1]
		info.text = v
		info.value = k
		info.checked = frame.currentValue == k
		info.func = DropDownOnClick
		info.arg1, info.arg2 = frame, k
		UIDropDownMenu_AddButton(info)
	end
end
--创建下拉菜单函数
function panel:CreateDropDown(text, valueList, LoadValue, SaveValue, defaultValue)
	local dropDown = CreateFrame('Frame', UniqueName('DropDown'), self, 'UIDropDownMenuTemplate')

	local title = self:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmallLeft')
	title:SetText(text)
	title:SetPoint('BOTTOMLEFT', dropDown, 'left', -80, -6)

	dropDown.LoadValue = LoadValue
	dropDown.SaveValue = SaveValue
	dropDown.defaultValue = defaultValue
	dropDown.UpdateValue = function(self)
		UIDropDownMenu_SetText(self, self.valueTexts[self.currentValue])
	end

	dropDown.valueList = valueList
	dropDown.valueTexts = {}
	for i=1,#valueList,2 do
		local k, v = valueList[i], valueList[i + 1]
		dropDown.valueTexts[k] = v
	end

	dropDown:SetScript('OnShow', function(self)
		UIDropDownMenu_Initialize(self, DropDownInitialize)
	end)

	UIDropDownMenu_JustifyText(dropDown, 'LEFT')
	UIDropDownMenu_SetWidth(dropDown, 120)
	UIDropDownMenu_SetButtonWidth(dropDown, 144)

	self.controls[dropDown:GetName()] = dropDown
	return dropDown
end
--设置面板初始化函数
function panel:Initialize()
	-- 独自一人时
	local solo = self:CreateHeading('独自一人')
	solo:SetPoint('TOPLEFT', title, 'BOTTOMLEFT', 0, -20)
	
	local soloOwnInterrupt = self:CreateDropDown(
		'自己打断',
		{ 'off', '关', 'say', '说', 'self', '自己可见' },
		function() return config.solo.ownInterrupt end,
		function(v) config.solo.ownInterrupt = v end,
		'self'
	)
	soloOwnInterrupt:SetPoint('TOPLEFT', solo, 'BOTTOMLEFT', 100, -10)

	local soloOwnDispel = self:CreateDropDown(
		'自己驱散',
		{ 'off', '关', 'say', '说', 'self', '自己可见' },
		function() return config.solo.ownDispel end,
		function(v) config.solo.ownDispel= v end,
		'self'
	)
	soloOwnDispel:SetPoint('TOPLEFT', solo, 'BOTTOMLEFT', 100, -30)
	
	local soloOwnStolen = self:CreateDropDown(
		'自己偷取',
		{ 'off', '关', 'say', '说', 'self', '自己可见' },
		function() return config.solo.ownStolen end,
		function(v) config.solo.ownStolen= v end,
		'self'
	)
	soloOwnStolen:SetPoint('TOPLEFT', solo, 'BOTTOMLEFT', 100, -50)
	
	local soloOwnReflect = self:CreateDropDown(
		'自己反射',
		{ 'off', '关', 'say', '说', 'self', '自己可见' },
		function() return config.solo.ownReflect end,
		function(v) config.solo.ownReflect= v end,
		'self'
	)
	soloOwnReflect:SetPoint('TOPLEFT', solo, 'BOTTOMLEFT', 100, -70)
	
	local soloOwnAD = self:CreateDropDown(
		'自己毒瘤',
		{ 'off', '关', 'say', '说', 'self', '自己可见' },
		function() return config.solo.ownAD end,
		function(v) config.solo.ownAD= v end,
		'self'
	)
	soloOwnAD:SetPoint('TOPLEFT', solo, 'BOTTOMLEFT', 100, -90)
	-- 副本队伍中
	local instance = self:CreateHeading('副本队伍中')
	instance:SetPoint('TOPLEFT', solo, 'BOTTOMLEFT', 0, -130)
	
	local instanceOwnInterrupt = self:CreateDropDown(
		'自己打断',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.ownInterrupt end,
		function(v) config.instance.ownInterrupt = v end,
		'self'
	)
	instanceOwnInterrupt:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -10)

	local instanceOwnDispel = self:CreateDropDown(
		'自己驱散',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.ownDispel end,
		function(v) config.instance.ownDispel= v end,
		'self'
	)
	instanceOwnDispel:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -30)
	
	local instanceOwnStolen = self:CreateDropDown(
		'自己偷取',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.ownStolen end,
		function(v) config.instance.ownStolen= v end,
		'self'
	)
	instanceOwnStolen:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -50)
	
	local instanceOwnReflect = self:CreateDropDown(
		'自己反射',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.ownReflect end,
		function(v) config.instance.ownReflect= v end,
		'self'
	)
	instanceOwnReflect:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -70)
	
	local instanceOwnAD = self:CreateDropDown(
		'自己毒瘤',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.ownAD end,
		function(v) config.instance.ownAD= v end,
		'self'
	)
	instanceOwnAD:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -90)
	
	local instanceOtherInterrupt = self:CreateDropDown(
		'他人打断',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.otherInterrupt end,
		function(v) config.instance.otherInterrupt = v end,
		'self'
	)
	instanceOtherInterrupt:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -110)

	local instanceOtherDispel = self:CreateDropDown(
		'他人驱散',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.otherDispel end,
		function(v) config.instance.otherDispel= v end,
		'self'
	)
	instanceOtherDispel:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -130)
	
	local instanceOtherStolen = self:CreateDropDown(
		'他人偷取',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.otherStolen end,
		function(v) config.instance.otherStolen= v end,
		'self'
	)
	instanceOtherStolen:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -150)
	
	local instanceOtherReflect = self:CreateDropDown(
		'他人反射',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.otherReflect end,
		function(v) config.instance.otherReflect= v end,
		'self'
	)
	instanceOtherReflect:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -170)
	
	local instanceOtherAD = self:CreateDropDown(
		'他人毒瘤',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'instance', '副本频道' },
		function() return config.instance.otherAD end,
		function(v) config.instance.otherAD= v end,
		'self'
	)
	instanceOtherAD:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 100, -190)
	--小队中
	local party = self:CreateHeading('小队中')
	party:SetPoint('TOPLEFT', instance, 'BOTTOMLEFT', 0, -230)
	
	local partyOwnInterrupt = self:CreateDropDown(
		'自己打断',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.ownInterrupt end,
		function(v) config.party.ownInterrupt = v end,
		'self'
	)
	partyOwnInterrupt:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -10)

	local partyOwnDispel = self:CreateDropDown(
		'自己驱散',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.ownDispel end,
		function(v) config.party.ownDispel= v end,
		'self'
	)
	partyOwnDispel:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -30)
	
	local partyOwnStolen = self:CreateDropDown(
		'自己偷取',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.ownStolen end,
		function(v) config.party.ownStolen= v end,
		'self'
	)
	partyOwnStolen:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -50)
	
	local partyOwnReflect = self:CreateDropDown(
		'自己反射',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.ownReflect end,
		function(v) config.party.ownReflect= v end,
		'self'
	)
	partyOwnReflect:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -70)
	
	local partyOwnAD = self:CreateDropDown(
		'自己毒瘤',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.ownAD end,
		function(v) config.party.ownAD= v end,
		'self'
	)
	partyOwnAD:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -90)
	
	local partyOtherInterrupt = self:CreateDropDown(
		'他人打断',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.otherInterrupt end,
		function(v) config.party.otherInterrupt = v end,
		'self'
	)
	partyOtherInterrupt:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -110)

	local partyOtherDispel = self:CreateDropDown(
		'他人驱散',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.otherDispel end,
		function(v) config.party.otherDispel= v end,
		'self'
	)
	partyOtherDispel:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -130)
	
	local partyOtherStolen = self:CreateDropDown(
		'他人偷取',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.otherStolen end,
		function(v) config.party.otherStolen= v end,
		'self'
	)
	partyOtherStolen:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -150)
	
	local partyOtherReflect = self:CreateDropDown(
		'他人反射',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.otherReflect end,
		function(v) config.party.otherReflect= v end,
		'self'
	)
	partyOtherReflect:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -170)
	
	local partyOtherAD = self:CreateDropDown(
		'他人毒瘤',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' },
		function() return config.party.otherAD end,
		function(v) config.party.otherAD= v end,
		'self'
	)
	partyOtherAD:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 100, -190)
	--团队中
	local raid = self:CreateHeading('团队中')
	raid:SetPoint('TOPLEFT', party, 'BOTTOMLEFT', 0, -230)
	
	local raidOwnInterrupt = self:CreateDropDown(
		'自己打断',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.ownInterrupt end,
		function(v) config.raid.ownInterrupt = v end,
		'self'
	)
	raidOwnInterrupt:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -10)

	local raidOwnDispel = self:CreateDropDown(
		'自己驱散',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.ownDispel end,
		function(v) config.raid.ownDispel= v end,
		'self'
	)
	raidOwnDispel:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -30)
	
	local raidOwnStolen = self:CreateDropDown(
		'自己偷取',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.ownStolen end,
		function(v) config.raid.ownStolen= v end,
		'self'
	)
	raidOwnStolen:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -50)
	
	local raidOwnReflect = self:CreateDropDown(
		'自己反射',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.ownReflect end,
		function(v) config.raid.ownReflect= v end,
		'self'
	)
	raidOwnReflect:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -70)
	
	local raidOwnAD = self:CreateDropDown(
		'自己毒瘤',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.ownAD end,
		function(v) config.raid.ownAD= v end,
		'self'
	)
	raidOwnAD:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -90)
	
	local raidOtherInterrupt = self:CreateDropDown(
		'他人打断',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.otherInterrupt end,
		function(v) config.raid.otherInterrupt = v end,
		'self'
	)
	raidOtherInterrupt:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -110)

	local raidOtherDispel = self:CreateDropDown(
		'他人驱散',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.otherDispel end,
		function(v) config.raid.otherDispel= v end,
		'self'
	)
	raidOtherDispel:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -130)
	
	local raidOtherStolen = self:CreateDropDown(
		'他人偷取',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.otherStolen end,
		function(v) config.raid.otherStolen= v end,
		'self'
	)
	raidOtherStolen:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -150)
	
	local raidOtherReflect = self:CreateDropDown(
		'他人反射',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.otherReflect end,
		function(v) config.raid.otherReflect= v end,
		'self'
	)
	raidOtherReflect:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -170)
	
	local raidOtherAD = self:CreateDropDown(
		'他人毒瘤',
		{ 'off', '关', 'say', '说', 'self', '自己可见', 'party', '小队频道' , 'raid', '团队频道' },
		function() return config.raid.otherAD end,
		function(v) config.raid.otherAD= v end,
		'self'
	)
	raidOtherAD:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 100, -190)
	--是否开启自动售卖
	local autoSellEnabled = self:CreateCheckBox(
		L["setAutoSell"],
		function() return config.autoSell end,
		function(v) config.autoSell = v end,
		true)
	autoSellEnabled:SetPoint('TOPLEFT', raid, 'BOTTOMLEFT', 0, -230)
end

--面板初始化
panel:Initialize()
panel:Show()
scrollFrame.okay = scrollFrame.ConfigOkay
scrollFrame.default = scrollFrame.ConfigDefault
scrollFrame.refresh = scrollFrame.ConfigRefresh
scrollFrame:ConfigRefresh()
scrollFrame:Show()





