local addonName, addon = ...
local L = addon.language
local avoidableDamage = addon.avoidableDamage

function addon:UpdateConfig(dst, src)
	for k, v in pairs(src) do
		if type(v) == 'table' then
			if type(dst[k]) == 'table' then
				self:UpdateConfig(dst[k], v)
			else
				dst[k] = self:UpdateConfig({}, v)
			end
		elseif type(dst[k]) ~= 'table' then
			dst[k] = v
		end
	end
	return dst
end

function addon:Announce(mode, msg, sound)
	if mode == "off" then
		return
	elseif mode == "self" or mode == "" then
		print(msg)
		return
	elseif mode == "instance" or mod == "i" then
		mode = "INSTANCE_CHAT"
	elseif mode == "s"then
		mode = "say"
	elseif mode == "g" then
		mode = "guild"
	elseif mode == "p" then
		mode = "party"
	elseif mode == "r" then
		mode = "raid"
	elseif mode == "y" then
		mode = "yell"
	end
	SendChatMessage(msg, mode:upper())
	if sound ~= '' then
		PlaySoundFile(sound, 'MASTER')
	end
end

function addon:Round(number, decimals)
    return (("%%.%df"):format(decimals)):format(number)
end

function addon:SendStats(mode)
	local N = "["..UnitName("player").."]"..L["statsReport"]
	local IL_C = {GetAverageItemLevel()}
	local IL = L["itemLevel"]..string.format("%.1f", IL_C[2])
		
	local S_C = UnitStat("player",1)
	local AG_C = UnitStat("player",2)
	local IN_C = UnitStat("player",4)
	if S_C > AG_C and S_C > IN_C then
		S = " "..L["Strength"]..S_C
	elseif AG_C > S_C and AG_C > IN_C then
		S = " "..L["agility"]..AG_C
	elseif IN_C > S_C and IN_C > AG_C then
		S = " "..L["intelligence"]..IN_C
	end
		
	local A_C = {UnitArmor("player")}
	local A = " "..L["amor"]..A_C[2]
	local CC = " "..L["crit"]..addon:Round(GetCritChance(), 1).."%"
	local H = " "..L["haste"]..addon:Round(GetHaste(), 1).."%"
	local M = " "..L["mastery"]..addon:Round(GetMasteryEffect(), 1).."%"
	local V  = " "..L["versatility"]..addon:Round(GetCombatRatingBonus(29), 1).."%"
	local LS  = " "..L["leech"]..addon:Round(GetCombatRatingBonus(17), 1).."%"
	local DC = " "..L["dodge"]..addon:Round(GetDodgeChance(), 1).."%"
	local PC = " "..L["parry"]..addon:Round(GetParryChance(), 1).."%"
	local BC = " "..L["block"]..addon:Round(GetBlockChance(), 1).."%"
	local END = " "..L["statsEnd"]
	local msg = N..IL..S..A..CC..H..M..V..LS..DC..PC..BC..END
	addon:Announce(mode, msg, "")
end

function addon:QueryAttacking()
	local h = (time()-1545170400)/3600
	local n = math.modf(h/19)%6+1
	local l = h%19
	local a ={L["Zuldazar"],L["TiragardeSound"],L["Nazmir"],L["StormsongValley"],L["Vol'dun"],L["Drustvar"] ,L["Zuldazar"]}
	print(l<7 and string.format(L["attacking"],a[n],7-l) or string.format(L["nextAttack"],a[n+1],19-l))
end

function addon:Count(a)
	local count = 0
	for k, v in pairs(a) do
		count = count + 1
	end
	return count
end

function addon:Sort(a,key)
	local count = addon:Count(a)
	if count < 2 then return a end
	local cache = {}
	for i = 1, count-1 do
		for j = 1, count - i do
			if a[j][key] < a[j+1][key] then
				cache = a[j]
				a[j] = a[j+1]
				a[j+1] = cache
			end
		end
	end
	return a
end

function addon:ShortName(name)
	local shortName = name:match("^(%S*)%-") or name or '?'
	return shortName
end

function addon:IsInMyGroup(flag)
	local inParty = IsInGroup()
	local inRaid = IsInRaid()
	local result = (inRaid and bit.band(flag, COMBATLOG_OBJECT_AFFILIATION_RAID) ~= 0) or (inParty and bit.band(flag, COMBATLOG_OBJECT_AFFILIATION_PARTY) ~= 0)
	return result
end

function addon:IsPlayer(flag)
	local result = bit.band(flag, COMBATLOG_OBJECT_TYPE_PLAYER) ~= 0
	return result
end

function addon:ClassColor(name)
	local _, classFileName = UnitClass(name)
	name = '|c' .. RAID_CLASS_COLORS[classFileName or 'PRIEST'].colorStr .. name .. '|r'
	return name
end

function addon:SaveDamage(destName, spellId, amount)
	if avoidableDamage["cache"][destName] == nil then
		avoidableDamage["cache"][destName] = {}
	end
	if avoidableDamage["saveByName"][destName] == nil then
		avoidableDamage["saveByName"][destName] = {}
	end
	if avoidableDamage["sum"][destName] == nil then
		avoidableDamage["sum"][destName] = 0
	end
	if avoidableDamage["saveBySpell"][spellId] == nil then
		avoidableDamage["saveBySpell"][spellId] = {}
	end
	if amount == nil then
		amount = 0
	end
	
	avoidableDamage["sum"][destName] = avoidableDamage["sum"][destName] + amount
	if avoidableDamage["cache"][destName][spellId] == nil then
		avoidableDamage["cache"][destName][spellId] = amount
	else
		avoidableDamage["cache"][destName][spellId] = avoidableDamage["cache"][destName][spellId] + amount
	end
	if avoidableDamage["saveByName"][destName][spellId] == nil then
		avoidableDamage["saveByName"][destName][spellId] = amount
	else
		avoidableDamage["saveByName"][destName][spellId] = avoidableDamage["saveByName"][destName][spellId] + amount
	end
	if avoidableDamage["saveBySpell"][spellId][destName] == nil then
		avoidableDamage["saveBySpell"][spellId][destName] = amount
	else
		avoidableDamage["saveBySpell"][spellId][destName] = avoidableDamage["saveBySpell"][spellId][destName] + amount
	end
end

function addon:AnnounceDamage(mode, destName)
	if avoidableDamage["timer"][destName] == nil then
		avoidableDamage["timer"][destName]  = true
		C_Timer.After(2,function()
			local msg = "["..destName.."] ← ["
			local amount = 0
			for k,v in pairs(avoidableDamage["cache"][destName]) do
				msg = msg..GetSpellLink(k).."]"
				amount = amount + v
			end
			avoidableDamage["cache"][destName] = nil
			avoidableDamage["timer"][destName] = nil
			local userMaxHealth = UnitHealthMax(destName)
			local msgAmount = addon:Round(amount/1000 ,1)
			local pct = Round(amount / userMaxHealth * 100)
			msg = msg.."("..msgAmount.."k - "..pct.."%)"
			table.insert(avoidableDamage["alist"], msg)
			if pct >= 20 then
				addon:Announce(mode, msg, "")
			end
		end)
	end
end

function addon:SpellDamage(mode, destName, spellId, amount)
	if (avoidableDamage["spells"][spellId] or (avoidableDamage["spellsNoTank"][spellId] and UnitGroupRolesAssigned(destName) ~= "TANK")) and UnitIsPlayer(destName) then
		addon:SaveDamage(destName, spellId, amount)
		addon:AnnounceDamage(mode, destName)
	end
end

function addon:AnnounceAD(mode)
	if addon:Count(avoidableDamage["sum"]) == 0 then
		addon:Announce(mode, "--------------------", "")
		addon:Announce(mode, "没有人受到可规避伤害，人人都是魂斗罗！", "")
		addon:Announce(mode, "--------------------", "")
		return
	end
	avoidableDamage["list"] = {}
	for k, v in pairs(avoidableDamage["sum"]) do
		table.insert(avoidableDamage["list"], {["name"] = k, ["damage"] = v })
	end
	avoidableDamage["list"] = addon:Sort(avoidableDamage["list"], "damage")
	addon:Announce(mode, "--------------------", "")
	addon:Announce(mode, "毒瘤排行榜", "")
	addon:Announce(mode, "--------------------", "")
	for k,v in pairs(avoidableDamage["list"]) do
		local msg = k..". "..v["name"].." "..addon:Round(v["damage"] / 1000 ,1).."k"
		addon:Announce(mode, msg, "")
	end
	addon:Announce(mode, "--------------------", "")
end

function addon:AnnounceADS(mode)
	if addon:Count(avoidableDamage["sum"]) == 0 then
		addon:Announce(mode, "--------------------", "")
		addon:Announce(mode, "没有人受到可规避伤害，人人都是魂斗罗！", "")
		addon:Announce(mode, "--------------------", "")
		return
	end
	local list = {}
	for id, v in pairs(avoidableDamage["saveBySpell"]) do
		local sum = 0
		for name, amount in pairs(v) do
			sum = sum + amount
		end
		table.insert(list, {["id"] = id, ["damage"] = sum })
	end
	list = addon:Sort(list, "damage")
	addon:Announce(mode, "--------------------", "")
	addon:Announce(mode, "可规避法术伤害排行榜", "")
	addon:Announce(mode, "--------------------", "")
	for k,v in pairs(list) do
		local msg = k..". "..GetSpellLink(v["id"]).." "..addon:Round(v["damage"] / 1000 ,1).."k"
		addon:Announce(mode, msg, "")
	end
	addon:Announce(mode, "--------------------", "")
end

function addon:AnnounceADN(mode)
	local name = avoidableDamage["list"][1]["name"]
	if not avoidableDamage["saveByName"][name] then
		print("["..name.."]未受到毒瘤伤害。")
		return
	end
	list = {}
	for id, v in pairs(avoidableDamage["saveByName"][name]) do
		table.insert(list, {["id"] = id, ["damage"] = v })
	end
	list = addon:Sort(list, "damage")
	addon:Announce(mode, "--------------------", "")
	addon:Announce(mode, "["..name.."]受到的可规避法术伤害排行榜", "")
	addon:Announce(mode, "--------------------", "")
	for k,v in pairs(list) do
		local msg = k..". "..GetSpellLink(v["id"]).." "..addon:Round(v["damage"] / 1000 ,1).."k"
		addon:Announce(mode, msg, "")
	end
	addon:Announce(mode, "--------------------", "")
end

function addon:AnnounceADMsgList(mode)
	for k, v in pairs(avoidableDamage["alist"]) do
		local msg = k..". "..v
		addon:Announce(mode, msg, "")
	end
end